# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2015 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from setuptools import setup, find_packages
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open(path.join(here, 'VERSION.txt'), encoding='utf-8') as f:
    version = f.read().strip()

setup(
    name='kansha',
    version=version,
    author='Net-ng',
    author_email='contact@net-ng.com',
    description='Manage and share collaborative pinboards on the web.',
    long_description=long_description,
    license='BSD',
    keywords='',
    url='',
    packages=find_packages(),
    include_package_data=True,
    package_data={'': ['*.cfg', '*.ini', '*.jpg']},
    zip_safe=False,
    install_requires=(
        'alembic',
        'Babel',
        'dateutils',
        'docutils',
        'flup==1.0.2',
        'identicon',
        'nagare[database,i18n]==0.4.1.post473',
        'nagare-services',
        'oauth2==1.5.211',
        'Paste',
        'Pillow',
        'pycrypto',
        'requests',
        'SQLAlchemy',
        'xlwt',
    ),
    dependency_links=[path.join(here, 'vendors'), 'http://www.nagare.org/snapshots/'],
    extras_require={'test': ('nose',),
                    'htmldocs': ('sphinx',),
                    'debug': ('WebError',),
                    'ldap': ('python-ldap',),
                    'postgres': ('psycopg2',),
                    'mysql': ('oursql',),
                    'elastic': ('elasticsearch',)},
    message_extractors={'kansha': [('**.py', 'python', None)]},
    entry_points="""
      [console_scripts]
      kansha-admin = kansha.app:run

      [kansha.commands]
      alembic-current = kansha.alembic.admin:AlembicCurrentCommand
      alembic-downgrade = kansha.alembic.admin:AlembicDowngradeCommand
      alembic-revision = kansha.alembic.admin:AlembicRevisionCommand
      alembic-stamp = kansha.alembic.admin:AlembicStampCommand
      alembic-upgrade = kansha.alembic.admin:AlembicUpgradeCommand
      create-index = kansha.batch.create_index:ReIndex
      save-config = kansha.batch.save_config:SaveConfig
      create-demo = kansha.batch.create_demo:CreateDemo

      [kansha.services]
      authentication = kansha.services.authentication_repository:AuthenticationsRepository
      mail_sender = kansha.services.mail:MailSender
      assets_manager = kansha.services.simpleassetsmanager.simpleassetsmanager:SimpleAssetsManager

      [kansha.authentication]
      dblogin = kansha.authentication.database.forms:Login
      oauthlogin = kansha.authentication.oauth.forms:Login
      ldaplogin = kansha.authentication.ldap.forms:Login

      [kansha.card.extensions]
      labels = kansha.card_addons.label.comp:CardLabels
      description = kansha.card_addons.description.comp:CardDescription
      checklists = kansha.card_addons.checklist.comp:Checklists
      gallery = kansha.card_addons.gallery.comp:Gallery
      comments = kansha.card_addons.comment.comp:Comments
      due_date = kansha.card_addons.due_date.comp:DueDate
      votes = kansha.card_addons.vote.comp:Votes
      weight = kansha.card_addons.weight:CardWeightEditor
      members = kansha.card_addons.members:CardMembers

      [nagare.applications]
      kansha = kansha.app:app

      [search.engines]
      dummy = kansha.services.search.dummyengine:DummySearchEngine
      sqlite = kansha.services.search.sqliteengine:SQLiteFTSEngine
      elastic = kansha.services.search.elasticengine:ElasticSearchEngine
      """
)



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Add due_date to cards

Revision ID: 537fa16b46e7
Revises: None
Create Date: 2013-09-10 16:44:45.986636

"""

# revision identifiers, used by Alembic.
revision = '537fa16b46e7'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('card', sa.Column('due_date', sa.Date))


def downgrade():
    op.drop_column('card', 'due_date')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import unittest

from kansha.services.search import schema, sqliteengine, elasticengine

#TODO: test all types on schema, doc creation and search

class MyDocument(schema.Document):
    title = schema.Text(stored=True)
    tags = schema.Text(stored=False)
    pages = schema.Int(stored=True)
    description = schema.Text
    price = schema.Float(stored=True, indexed=False)


class Person(schema.Document):
    firstname = schema.Keyword(stored=True)
    lastname = schema.Keyword(stored=True)
    height = schema.Float
    weight = schema.Float
    age = schema.Int(stored=True, indexed=False)


class TestSchema(unittest.TestCase):

    def setUp(self):

        self.Doc = MyDocument

    def test_class_attributes(self):
        self.assertIsInstance(self.Doc.title, schema.Text)
        self.assertEqual(self.Doc.price.indexed, False)
        self.assertIsInstance(self.Doc.description, schema.Text)
        self.assertEqual(self.Doc.title.schema, self.Doc)
        self.assertEqual(self.Doc.pages.name, 'pages')

    def test_instance_default_attributes(self):
        doc = self.Doc(1)
        self.assertEqual(
            doc.title, schema.Text.default, u'default was not set')
        self.assertEqual(doc.pages, schema.Int.default, u'default was not set')

    def test_instance_attributes(self):
        doc = self.Doc('docid', title=u'Titre', tags=u'mot livre français',
                       pages=89, description=u'Description', price=5.6)
        self.assertEqual(doc.title, u'Titre', u'attribute not set')
        self.assertEqual(doc.tags, u'mot livre français', u'attribute not set')
        self.assertEqual(doc.pages, 89, u'attribute not set')
        self.assertEqual(doc.description, u'Description', u'attribute not set')
        self.assertEqual(doc.price, 5.6, u'attribute not set')


class TestImpSchema(TestSchema):

    def setUp(self):
        TestDocument = schema.Schema('MyDocument')
        TestDocument.add_field(schema.Text('title', stored=True))
        TestDocument.add_field(schema.Text('tags', stored=False))
        TestDocument.add_field(schema.Int('pages', stored=True))
        TestDocument.add_field(schema.Text('description'))
        TestDocument.add_field(schema.Float('price', stored=True, indexed=False))
        self.Doc = TestDocument


class TestImpSchemaAltSyntax(TestSchema):

    def setUp(self):
        TestDocument = (
            schema.Schema('MyDocument') +
            schema.Text('title', stored=True) +
            schema.Text('tags', stored=False) +
            schema.Int('pages', stored=True) +
            schema.Text('description') +
            schema.Float('price', stored=True, indexed=False)
        )
        self.Doc = TestDocument


class TestImpSchemaConstructor(TestSchema):

    def setUp(self):
        TestDocument = schema.Schema(
            'MyDocument',
            schema.Text('title', stored=True),
            schema.Text('tags', stored=False),
            schema.Int('pages', stored=True),
            schema.Text('description'),
            schema.Float('price', stored=True, indexed=False)
        )
        self.Doc = TestDocument


class IndexCursor(object):

    def __init__(self, testcase):
        self.testcase = testcase
        self.schema = ''
        self.docid = 0
        self.fields = {}
        self.operation = ''

    def insert(self, schema_name, docid, **fields):
        self._action('insert', schema_name, docid, fields)

    def update(self, schema_name, docid, **fields):
        self._action('update', schema_name, docid, fields)

    def _action(self, operation, schema_name, docid, fields):
        self.schema = schema_name
        self.docid = docid
        self.fields = fields
        self.operation = operation

    def assert_match(self, operation, document):
        self.testcase.assertEqual(operation, self.operation)
        self.testcase.assertEqual(document.schema_name, self.schema)
        self.testcase.assertEqual(document._id, self.docid)
        for name, value in self.fields.iteritems():
            self.testcase.assertEqual(getattr(document, name), value)


class TestIndexableAPI(unittest.TestCase):

    def setUp(self):
        TestDocument = (
            schema.Schema('MyDocument') +
            schema.Text('title', stored=True) +
            schema.Text('tags', stored=False) +
            schema.Int('pages', stored=True) +
            schema.Text('description') +
            schema.Float('price', stored=True, indexed=False)
        )
        self.doc = TestDocument('docid', title=u'Titre', tags=u'mot livre français',
                       pages=89, description=u'Description', price=5.6)

    def test_insert(self):
        cursor = IndexCursor(self)
        self.doc.save(cursor)
        cursor.assert_match('insert', self.doc)

    def test_update(self):
        cursor = IndexCursor(self)
        self.doc.save(cursor, update=True)
        cursor.assert_match('update', self.doc)


class SearchTestCase(object):

    collection = u'test'

    def _create_search_engine(self):
        raise NotImplementedError()

    def setUp(self):
        self.engine = self._create_search_engine()
        self.engine.create_collection([MyDocument, Person])
        self.MyDocument = MyDocument
        self.Person = Person

    def tearDown(self):
        self.engine.delete_collection()

    def load_documents(self):
        '''helper function'''
        doc1 = self.MyDocument(
            'doc1', title=u'Titre', tags=u'Un livre français best seller',
            pages=89, description=u'Description de qualité, avec des services.', price=10.0)
        doc2 = self.MyDocument(
            'doc2', title=u'Unit tests', tags=u'quality software',
            pages=89, description=u'Best practices and services.', price=9.90)
        doc3 = self.Person(
            'p1', firstname=u'John', lastname=u'Doe', height=180,
            weight=80.5, age=40)
        doc4 = self.Person(
            'p2', firstname=u'Joan', lastname=u'Watson', height=170,
            weight=60, age=42)
        self.engine.add_document(doc1)
        self.engine.add_document(doc2)
        self.engine.add_document(doc3)
        self.engine.add_document(doc4)
        self.engine.commit(sync=True)

    def test_add_document(self):
        self.load_documents()

    def test_remove_document(self):
        self.load_documents()
        self.engine.delete_document(self.MyDocument, 'doc1')
        self.engine.commit(sync=True)
        query = self.MyDocument.match(u'best')
        res = self.engine.search(query)
        self.assertEqual(len(res), 1)

    def test_update_document(self):
        self.load_documents()
        doc = self.MyDocument.delta('doc1', title=u'Titre augmenté')
        self.engine.update_document(doc)
        self.engine.commit(sync=True)
        query = self.MyDocument.match(u'augmenté')
        res = self.engine.search(query)
        self.assertEqual(len(res), 1)
        assert(res[0][1].title == u'Titre augmenté'
               and res[0][1].price == 10.0)

    def test_match_all(self):
        self.load_documents()
        query = self.MyDocument.match(u'best')
        res = self.engine.search(query)
        self.assertEqual(len(res), 2)
        query = self.MyDocument.match(u'tests')
        res = self.engine.search(query)
        self.assertEqual(res[0][1]._id, 'doc2')
        self.assertEqual(res[0][1].title, u'Unit tests')
        self.assertIsNone(res[0][1].tags)  # not stored

    def test_match_field(self):
        self.load_documents()
        query = self.MyDocument.tags.match(u'best')
        res = self.engine.search(query)
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][1].title, u'Titre')

    def test_equal_field(self):
        self.load_documents()
        res = self.engine.search(self.MyDocument.pages == 89)
        self.assertEqual(len(res), 2)
        res = self.engine.search(self.Person.lastname == u'Doe')
        self.assertEqual(len(res), 1)

    def test_unindexed_field(self):
        '''Would fails on SQLite for operators other than match'''
        self.load_documents()
        res = self.engine.search(self.Person.age == 40)
        if not isinstance(self.engine, sqliteengine.SQLiteFTSEngine):
            self.assertEqual(len(res), 0)  # age not indexed

    def test_partial_match(self):
        self.load_documents()
        res = self.engine.search(self.MyDocument.match(u'qualit'))
        self.assertEqual(len(res), 2)

    def test_disjonction(self):
        '''Fails on SQLite if both use match'''
        self.load_documents()
        query = (self.MyDocument.pages == 89) | self.MyDocument.description.match(
            u'practices')
        res = self.engine.search(query)
        self.assertEqual(len(res), 2)

    def test_disjonction_fail(self):
        self.assertRaises(AssertionError, lambda: (
            self.MyDocument.pages == 89) | self.Person.firstname.match(u'John'))

    def test_multiterm(self):
        '''All terms must be present'''
        self.load_documents()
        query = self.MyDocument.match(u'quality services')
        res = self.engine.search(query)
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][1]._id, 'doc2')
        query = self.MyDocument.match(u'quality description')
        res = self.engine.search(query)
        self.assertEqual(len(res), 0)

    def test_special_chars(self):
        self.load_documents()
        query = self.MyDocument.match('"best')
        res = self.engine.search(query)
        self.assertEqual(len(res), 2)


class ImpSchemaSearchTestCase(SearchTestCase):

    def setUp(self):
        MyDocument_ = schema.Schema('MyDocument')
        MyDocument_.add_field(schema.Text('title', stored=True))
        MyDocument_.add_field(schema.Text('tags', stored=False))
        MyDocument_.add_field(schema.Int('pages', stored=True))
        MyDocument_.add_field(schema.Text('description'))
        MyDocument_.add_field(schema.Float('price', stored=True, indexed=False))

        Person_ = schema.Schema('Person')
        Person_.add_field(schema.Keyword('firstname', stored=True))
        Person_.add_field(schema.Keyword('lastname', stored=True))
        Person_.add_field(schema.Float('height'))
        Person_.add_field(schema.Float('weight'))
        Person_.add_field(schema.Int('age', stored=True, indexed=False))

        self.engine = self._create_search_engine()
        self.engine.create_collection([MyDocument_, Person_])
        self.MyDocument = MyDocument_
        self.Person = Person_


class TestSQLiteEngine(SearchTestCase, unittest.TestCase):

    def _create_search_engine(self):
        return sqliteengine.SQLiteFTSEngine(self.collection, u'/tmp')


class TestElasticEngine(SearchTestCase, unittest.TestCase):

    def _create_search_engine(self):
        try:
            return elasticengine.ElasticSearchEngine(self.collection)
        except ValueError as exc:
            self.skipTest(unicode(exc))


class TestSQLiteEngineImpSchema(ImpSchemaSearchTestCase, unittest.TestCase):

    def _create_search_engine(self):
        return sqliteengine.SQLiteFTSEngine(self.collection, u'/tmp')


class TestElasticEngineImpSchema(ImpSchemaSearchTestCase, unittest.TestCase):

    def _create_search_engine(self):
        try:
            return elasticengine.ElasticSearchEngine(self.collection)
        except ValueError as exc:
            self.skipTest(unicode(exc))



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import unittest

from nagare import database
from elixir import metadata as __metadata__

from kansha import helpers
from kansha.services.actionlog import DummyActionLog


class CardExtensionTestCase(unittest.TestCase):

    extension_name = ''
    extension_class = None

    @property
    def card_copy(self):
        new_card = self.column.create_card(self.card.get_title())
        new_card.update(self.card)
        return new_card

    @property
    def extension_copy(self):
        new_card = self.card_copy
        new_extension = dict(new_card.extensions)[self.extension_name]() if self.extension_name else None
        return new_extension

    def setUp(self):
        database.set_metadata(__metadata__, 'sqlite:///:memory:', False, {})
        helpers.setup_db(__metadata__)
        helpers.set_context(helpers.create_user())
        extensions = [(self.extension_name, self.extension_class)] if self.extension_name else []
        self.board = board = helpers.create_board(extensions)
        self.column = column = board.create_column(1, u'test')
        self.card = column.create_card(u'test')
        self.extension = dict(self.card.extensions)[self.extension_name]() if self.extension_name else None

    def tearDown(self):
        helpers.teardown_db(__metadata__)


class DummyParent(object):
    def __init__(self):
        self.action_log = DummyActionLog()
        self.data = None



#-*- coding: utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

#=-
# (C)opyright PagesJaunes 2011
#
# This is Pagesjaunes proprietary source code
# Any reproduction modification or use without prior written
# approval from PagesJaunes is strictly forbidden.
#=-
import random
import string
from datetime import datetime, timedelta

from nagare import local, security
from nagare.database import session

from kansha.user import usermanager
from kansha.board import boardsmanager
from kansha.security import SecurityManager
from kansha.card.models import DataCard
from kansha.card_addons.vote import DataVote
from kansha.board import models as board_models
from kansha.services.mail import DummyMailSender
from kansha.services.search.dummyengine import DummySearchEngine
from kansha.services.services_repository import ServicesRepository
from kansha.services.dummyassetsmanager.dummyassetsmanager import DummyAssetsManager
from kansha.services.components_repository import CardExtensions


def setup_db(metadata):
    """Setup the tables

    In:
        - ``metadata`` -- the metadata from models
    """
    metadata.create_all()


def teardown_db(metadata):
    """Drop the tables

    In:
        - ``metadata`` -- the metadata from models
    """
    metadata.drop_all()
    session.close()


def word(length=20):
    """Random string generator

    In:
        - ``length`` -- the length of the string
    """
    return u''.join(random.sample(string.ascii_letters, length))


class DummySecurityManager(security.common.Rules):

    """
    Dummy Security Manager to be used in unit tests
    """

    def has_permission(self, user, perm, subject):
        return True


def set_dummy_context():
    """Set a dummy context for security permission checks
    """
    local.request = local.Thread()
    security.set_user(None)
    security.set_manager(DummySecurityManager())


def set_context(user=None):
    """ """
    local.request = local.Thread()
    security.set_user(user)
    security.set_manager(SecurityManager('somekey'))


def get_or_create_data_user(suffixe=''):
    '''Get test user for suffixe, or create if not exists'''
    user_test = usermanager.UserManager.get_by_username(
        u'usertest_%s' % suffixe)
    if not user_test:
        user_test = usermanager.UserManager().create_user(
            u'usertest_%s' % suffixe,
            u'password', u'User Test %s' % suffixe,
            u'user_test%s@net-ng.com' % suffixe)
        session.add(user_test)
    return user_test


def create_user(suffixe=''):
    """Create Test user
    """
    user_test = get_or_create_data_user(suffixe)
    return usermanager.UserManager.get_app_user(u'usertest_%s' % suffixe)


def create_services():
    'Service mockups for testing components'
    _services = ServicesRepository()
    _services.register('assets_manager', DummyAssetsManager())
    _services.register('mail_sender', DummyMailSender())
    _services.register('search_engine', DummySearchEngine(None))
    return _services


def get_boards_manager(extensions=[]):
    services = create_services()
    card_extensions = CardExtensions()
    for name, extension in extensions:
        card_extensions.register(name, extension)
    return boardsmanager.BoardsManager('', '', '', card_extensions, DummySearchEngine(None), services)


def create_board(card_extensions=[]):
    """Create boards with default columns and default cards
    """
    user = create_user()
    template = board_models.create_template_todo()
    boards_manager = get_boards_manager(card_extensions)
    # the user is automatically set to manager
    board = boards_manager.create_board_from_template(template.id, user)
    create_default_cards(board.data, user)
    board.set_title(word())
    board.load_children()
    return board


def create_default_cards(board, user):
    user = user.data
    green = board.get_label_by_title(u'Green')
    red = board.get_label_by_title(u'Red')
    column_1 = board.columns[0]
    cards = [DataCard(title=u"Welcome to your board!", creation_date=datetime.utcnow()),
             DataCard(title=u"We've created some lists and cards for you, so you can play with it right now!", creation_date=datetime.utcnow()),
             DataCard(title=u"Use color-coded labels for organization",
                      labels=[green, red], creation_date=datetime.utcnow()),
             DataCard(title=u"Make as many lists as you need!",
                      votes=[DataVote(user=user)], creation_date=datetime.utcnow()),
             DataCard(title=u"Try dragging cards anywhere.", creation_date=datetime.utcnow()),
             DataCard(title=u"Finished with a card? Delete it.", creation_date=datetime.utcnow()),
             ]
    for i, c in enumerate(cards):
        c.index = i
    column_1.cards = cards
    column_1.nb_max_cards = len(cards)

    column_2 = board.columns[1]
    cards = [DataCard(title=u'This is a card.', creation_date=datetime.utcnow()),
             DataCard(title=u"Click on a card to see what's behind it.", creation_date=datetime.utcnow()),
             DataCard(title=u"You can add files to a card.", creation_date=datetime.utcnow()),
             DataCard(
                 title=u'To learn more tricks, check out the manual.', creation_date=datetime.utcnow()),
             DataCard(title=u"Use as many boards as you want.", creation_date=datetime.utcnow())]
    for i, c in enumerate(cards):
        c.index = i
    column_2.cards = cards
    column_2.nb_max_cards = len(cards) + 2
    session.refresh(board)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import using_options
from elixir import ManyToMany, ManyToOne
from elixir import Field, Integer, DateTime, UnicodeText
from nagare.database import session
import datetime

from kansha.models import Entity


class DataCard(Entity):
    '''Card mapper'''

    using_options(tablename='card')
    title = Field(UnicodeText)
    index = Field(Integer)
    creation_date = Field(DateTime, default=datetime.datetime.utcnow)
    column = ManyToOne('DataColumn')

    def update(self, other):
        self.title = other.title
        self.index = other.index
        session.flush()

    @property
    def archived(self):
        return self.column.archive


class DummyDataCard(object):

    def __init__(self, title='dummy card', creation_date=datetime.datetime.utcnow()):
        self.title = title
        self.creation_date = creation_date
        self.index = 0

    def update(self, other):
        print 'update!'

    @property
    def archived(self):
        return False



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import absolute_import

import pickle

from nagare import database

# if objgraph is installed, it renders a graph that can help finding why
# the object is pickled
try:
    import objgraph
except ImportError:
    objgraph = None


class UnpicklableMixin(object):

    def __reduce__(self):
        if objgraph:
            database.session.expire_all()
            objgraph.show_backrefs([self], max_depth=5)

        raise pickle.PicklingError(
            'This object is not picklable: {!r}'.format(self)
        )



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--


from nagare.admin import util, command


class SaveConfig(command.Command):

    desc = 'Create a configuration template for kansha, from the default config into the current folder as kansha.cfg.'

    @staticmethod
    def run(parser, options, args):

        application = 'kansha'

        (cfgfile, app, dist, conf) = util.read_application(application,
                                                           parser.error)

        infile = open(cfgfile)
        outfile = open('kansha.cfg', 'w')
        outfile.write('datadir = "/path/to/folder"\n\n')
        for line in infile:
            oline = line.replace('$root/data', '$datadir')
            outfile.write(oline)
        infile.close()
        outfile.close()



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import json
import urllib
import urllib2
import urlparse

import oauth2 as oauth

from nagare import presentation


class OAuth1(object):
    name = 'OAuth1 provider'

    def __init__(self, key, secret, scopes=(), timeout=None):
        self.consumer = oauth.Consumer(key, secret)
        self.timeout = timeout
        self.scopes = scopes
        self.token = None

    def fetch(self, url, post=False, **kw):
        client = oauth.Client(self.consumer, self.token, None, self.timeout)
        headers, data = client.request(url,
                                       method='POST' if post else 'GET',
                                       body=urllib.urlencode(kw))
        if headers['content-type'].startswith(('application/json', 'text/javascript')):
            data = json.loads(data)

        return data

    def get_auth_url(self, callback_url, **kw):
        self.token = None
        data = self.fetch(self.request_token_endpoint,
                          oauth_callback=callback_url,
                          post=True, **kw)
        self.token = oauth.Token.from_string(data)
        return (self.authorization_endpoint + '?' +
                urllib.urlencode({'oauth_token': self.token.key}))

    def get_token(self, token, verifier):
        if not token:
            return None

        if verifier:
            self.token.set_verifier(verifier)
        data = self.fetch(self.token_endpoint, post=True)
        self.token = oauth.Token.from_string(data)
        return self

    def get_raw_profile(self):
        return self.fetch(self.profile_endpoint)

    def get_profile(self):
        return {}, self.get_raw_profile()


@presentation.render_for(OAuth1)
def render(self, h, comp, *args):
    action = lambda request, response: comp.answer(self.get_token(request.params.get('oauth_token'),
                                                                  request.params.get('oauth_verifier')))
    callback = h.a.action(action, with_request=True).get('href')

    h.response.status = 301
    h.response.headers['location'] = self.get_auth_url(h.request.relative_url(callback))
    return ''

# -----------------------------------------------------------------------------


class OAuth2(object):
    name = 'OAuth2 provider'

    def __init__(self, key, secret, scopes=(), timeout=None):
        self.key = key
        self.secret = secret
        self.scopes = scopes
        self.timeout = timeout

        self.callback_url = self.code = self.access_token = None

    def fetch(self, url, post=False, headers=None, **kw):
        params = urllib.urlencode(kw)
        if params and not post:
            url += ('?' + params)

        request = urllib2.Request(url, params if post else None, headers or {})

        if self.timeout:
            response = urllib2.urlopen(request, timeout=self.timeout)
        else:
            response = urllib2.urlopen(request)

        content_type = response.info().getheader('content-type').split(';')[0]
        data = response.read()
        response.close()

        if response.getcode() != 200:
            return None
        return json.loads(data)if content_type in ('application/json', 'text/javascript') else dict(urlparse.parse_qsl(data))

    def get_auth_url(self, callback_url, **kw):
        self.callback_url = callback_url

        kw['client_id'] = self.key
        kw['redirect_uri'] = callback_url
        if self.scopes:
            kw['scope'] = ' '.join(self.scopes)
        return '%s?%s' % (self.authorization_endpoint, urllib.urlencode(kw))

    def extract_token(self, code):
        if not code:
            return None
        self.code = code
        self.access_token = self.get_token(code)
        return self

    def get_token(self, code, **kw):
        response = self.fetch(
            self.token_endpoint,
            code=code,
            client_id=self.key, client_secret=self.secret,
            redirect_uri=self.callback_url, post=True, **kw
        )
        return response['access_token']  # , response.get('refresh_token')

    def get_raw_profile(self):
        return self.fetch(self.profile_endpoint, access_token=self.access_token)

    def get_profile(self):
        return {}, self.get_raw_profile()


@presentation.render_for(OAuth2)
def render(self, h, comp, *args):
    action = lambda request, response: comp.answer(self.extract_token(request.params.get('code')))
    callback = h.a.action(action, with_request=True).get('href')
    callback, state = callback.split('/?')
    h.response.status = 301
    h.response.headers['location'] = self.get_auth_url(h.request.relative_url(callback), state=state)
    return ''

# -----------------------------------------------------------------------------


class OpenIDConnect(OAuth2):
    name = 'OpenID connect provider'

    def get_auth_url(self, callback_url, **kw):
        return super(OpenIDConnect, self).get_auth_url(callback_url, response_type='code', **kw)

    def get_token(self, code):
        return super(OpenIDConnect, self).get_token(code, grant_type='authorization_code')

# -----------------------------------------------------------------------------


class Google(OpenIDConnect):
    name = 'google'

    authorization_endpoint = 'https://accounts.google.com/o/oauth2/auth'
    token_endpoint = 'https://accounts.google.com/o/oauth2/token'

    profile_endpoint = 'https://www.googleapis.com/oauth2/v1/userinfo'

    def __init__(self, key, secret, scopes=(), offline=False, prompt=False):
        super(Google, self).__init__(key, secret, ('openid',) + tuple(scopes))

        self.offline = offline
        self.prompt = prompt

    def fetch(self, url, post=False, **kw):
        response = super(Google, self).fetch(url, post=post, **kw)
        error = response.get('error')
        if error:
            raise IOError(error)

        return response

    def get_auth_url(self, callback_url, **kw):
        return super(Google, self).get_auth_url(
            callback_url,
            access_type='offline' if self.offline else 'online',
            approval_prompt='force' if self.prompt else 'auto',
            **kw
        )

    def get_profile(self):
        _, profile = super(Google, self).get_profile()
        return {
            'id': profile['id'],
            'name': profile.get('name', profile['email']),
            'email': profile.get('email'),
            'picture': profile.get('picture', None)
        }, profile


class Twitter(OAuth1):
    name = 'twitter'

    request_token_endpoint = 'https://api.twitter.com/oauth/request_token'
    authorization_endpoint = 'https://api.twitter.com/oauth/authorize'
    token_endpoint = 'https://api.twitter.com/oauth/access_token'

    profile_endpoint = 'https://api.twitter.com/1.1/account/verify_credentials.json'

    def get_profile(self):
        _, profile = super(Twitter, self).get_profile()
        return {
            'id': profile['id_str'],
            'name': profile['name'],
            'email': profile.get('email'),
            'picture': profile['profile_image_url_https']
        }, profile


class Facebook(OAuth2):
    name = 'facebook'

    authorization_endpoint = 'https://www.facebook.com/dialog/oauth'
    token_endpoint = 'https://graph.facebook.com/oauth/access_token'

    profile_endpoint = 'https://graph.facebook.com/me'

    # def __init__(self, key, secret, scopes=()):
    #     scopes = ['public_profile' if s == 'profile' else s for s in scopes]
    #     super(Facebook, self).__init__(key, secret, scopes)

    def get_raw_profile(self):
        return self.fetch(self.profile_endpoint, access_token=self.access_token, fields='id,name,email')

    def get_profile(self):

        _, profile = super(Facebook, self).get_profile()
        return {
            'id': profile['id'],
            'name': profile['name'],
            'email': profile.get('email'),
            'picture': self.get_picture_url()
        }, profile

    def get_picture_url(self):
        """Get URL picture"""
        profile_picture = self.fetch(self.profile_endpoint, post=False,
                                     fields='picture', access_token=self.access_token)
        if not(profile_picture['picture']['data']['is_silhouette']):
            return profile_picture['picture']['data']['url']
        else:
            return None


class Github(OAuth2):
    name = 'github'

    authorization_endpoint = 'https://github.com/login/oauth/authorize'
    token_endpoint = 'https://github.com/login/oauth/access_token'

    profile_endpoint = 'https://api.github.com/user'
    email_endpoint = 'https://api.github.com/user/emails'

    def get_email(self):
        resp = self.fetch(self.email_endpoint, post=False,
                             access_token=self.access_token)
        for email in resp:
            if email['primary']:
                return email['email']
        return ''

    def get_profile(self):
        _, profile = super(Github, self).get_profile()
        return {
            'id': str(profile['id']),
            'name': profile['name'],
            'email': self.get_email(),
            'picture': profile['avatar_url']
        }, profile


class Dropbox(OAuth1):
    name = 'dropbox'

    request_token_endpoint = 'https://api.dropbox.com/1/oauth/request_token'
    authorization_endpoint = 'https://www.dropbox.com/1/oauth/authorize'
    token_endpoint = 'https://api.dropbox.com/1/oauth/access_token'

    profile_endpoint = 'https://api.dropbox.com/1/account/info'

    def get_auth_url(self, callback_url):
        return super(Dropbox, self).get_auth_url(callback_url) + '&' + urllib.urlencode({'oauth_callback': callback_url})

    def get_profile(self):
        _, profile = super(Dropbox, self).get_profile()
        return {
            'id': str(profile['uid']),
            'name': profile['display_name'],
            'email': profile['email'],
            'picture': None
        }, profile


class Salesforce(OpenIDConnect):
    name = 'salesforce'

    authorization_endpoint = 'https://login.salesforce.com/services/oauth2/authorize'
    token_endpoint = 'https://login.salesforce.com/services/oauth2/token'

    def fetch(self, url, post=False, headers=None, **kw):
        data = super(Salesforce, self).fetch(url, post=post, headers=headers, **kw)
        self.profile_endpoint = data['id']
        return data

    def get_raw_profile(self):
        return self.fetch(self.profile_endpoint, headers={'Authorization': 'Bearer ' + self.access_token})

    def get_profile(self):
        _, profile = super(Salesforce, self).get_profile()
        return {
            'id': profile['user_id'],
            'name': profile['display_name'],
            'email': profile['email'],
            'picture': profile['photos']['picture']
        }, profile


class Flickr(OAuth1):
    name = 'flickr'

    request_token_endpoint = 'http://www.flickr.com/services/oauth/request_token'
    authorization_endpoint = 'http://www.flickr.com/services/oauth/authorize'
    token_endpoint = 'http://www.flickr.com/services/oauth/access_token'

    profile_endpoint = 'http://api.flickr.com/services/rest?method=flickr.test.login&format=json&nojsoncallback=1'

    def get_auth_url(self, callback_url):
        return super(Flickr, self).get_auth_url(callback_url) + '&perms=read'

    def get_raw_profile(self):
        user_id = super(Flickr, self).get_raw_profile()['user']['id']
        return self.fetch('http://api.flickr.com/services/rest?method=flickr.people.getInfo&format=json&nojsoncallback=1&user_id=' + user_id)

    def get_profile(self):
        _, profile = super(Flickr, self).get_profile()
        profile = profile['person']

        iconfarm = profile['iconfarm']
        iconserver = profile['iconserver']
        picture = 'http://farm%d.staticflickr.com/%s/buddyicons/%s.jpg' % (iconfarm, iconserver, profile['nsid'])

        return {
            'id': profile['id'],
            'name': profile['realname']['_content'] or profile['username']['_content'],
            'email': None,
            'picture': picture if (iconfarm or iconserver) else None
        }, profile


class Vimeo(OAuth1):
    name = 'vimeo'

    request_token_endpoint = 'https://vimeo.com/oauth/request_token'
    authorization_endpoint = 'https://vimeo.com/oauth/authorize'
    token_endpoint = 'https://vimeo.com/oauth/access_token'

    profile_endpoint = 'http://vimeo.com/api/rest/v2?method=vimeo.people.getInfo&format=json'

    def get_profile(self):
        _, profile = super(Vimeo, self).get_profile()
        profile = profile['person']
        return {
            'id': profile['id'],
            'name': profile['display_name'],
            'email': None,
            'picture': profile['portraits']['portrait'][1]['_content']
        }, profile


class Bitbucket(OAuth1):
    name = 'bitbucket'

    request_token_endpoint = 'https://bitbucket.org/!api/1.0/oauth/request_token'
    authorization_endpoint = 'https://bitbucket.org/!api/1.0/oauth/authenticate'
    token_endpoint = 'https://bitbucket.org/!api/1.0/oauth/access_token'

    profile_endpoint = 'https://api.bitbucket.org/1.0/user'

    def get_profile(self):
        _, profile = super(Bitbucket, self).get_profile()
        profile = profile['user']
        return {
            'id': profile['resource_uri'],
            'name': profile['first_name'] + ' ' + profile['last_name'],
            'email': None,
            'picture': profile['avatar']
        }, profile


class Yahoo(OAuth1):
    name = 'yahoo'

    request_token_endpoint = 'https://api.login.yahoo.com/oauth/v2/get_request_token'
    authorization_endpoint = 'https://api.login.yahoo.com/oauth/v2/request_auth'
    token_endpoint = 'https://api.login.yahoo.com/oauth/v2/get_token'

    profile_endpoint = 'http://social.yahooapis.com/v1/me/guid?format=json'

    def get_raw_profile(self):
        user_id = super(Yahoo, self).get_raw_profile()['guid']['value']
        return self.fetch('http://social.yahooapis.com/v1/user/%s/profile?format=json' % user_id)

    def get_profile(self):
        _, profile = super(Yahoo, self).get_profile()
        profile = profile['profile']

        given_name = profile.get('givenName', '')
        family_name = profile.get('familyName', '')
        emails = profile.get('emails')

        return {
            'id': profile['guid'],
            'name': ' '.join((given_name, family_name)) if (given_name or family_name) else profile['nickname'],
            'email': emails[0]['handle'] if emails else None,
            'picture': profile['image']['imageUrl']
        }, profile


class Dailymotion(OpenIDConnect):
    name = 'dailymotion'

    authorization_endpoint = 'https://api.dailymotion.com/oauth/authorize'
    token_endpoint = 'https://api.dailymotion.com/oauth/token'

    def fetch(self, url, post=False, headers=None, **kw):
        data = super(Dailymotion, self).fetch(url, post=post, headers=headers, **kw)
        self.user_id = data.get('uid')
        return data

    def get_raw_profile(self):
        return self.fetch('https://api.dailymotion.com/user/' + self.user_id)

    def get_profile(self):
        _, profile = super(Dailymotion, self).get_profile()
        return {
            'id': str(profile['id']),
            'name': profile['screenname'],
            'email': None,
            'picture': None
        }, profile


class Viadeo(OpenIDConnect):
    name = 'viadeo'

    authorization_endpoint = 'https://secure.viadeo.com/oauth-provider/authorize2'
    token_endpoint = 'https://secure.viadeo.com/oauth-provider/access_token2'

    profile_endpoint = 'https://api.viadeo.com/me'

    def get_profile(self):
        _, profile = super(Viadeo, self).get_profile()
        return {
            'id': str(profile['id']),
            'name': profile['name'],
            'email': None,
            'picture': profile['picture_small'] if profile['has_picture'] else None
        }, profile


class Linkedin(OAuth1):
    name = 'linkedin'

    request_token_endpoint = 'https://api.linkedin.com/uas/oauth/requestToken'
    authorization_endpoint = 'https://api.linkedin.com/uas/oauth/authorize'
    token_endpoint = 'https://api.linkedin.com/uas/oauth/accessToken'

    profile_endpoint = 'http://api.linkedin.com/v1/people/~:(id,firstName,lastName,email-address,picture-url)?format=json'

    def get_auth_url(self, callback_url):
        return super(Linkedin, self).get_auth_url(callback_url.replace('&', ';'), scope=' '.join(self.scopes))

    def get_profile(self):
        _, profile = super(Linkedin, self).get_profile()
        return {
            'id': profile['id'],
            'name': profile['firstName'] + ' ' + profile['lastName'],
            'email': profile['emailAddress'],
            'picture': profile['pictureUrl']
        }, profile


class Foursquare(OpenIDConnect):
    name = 'foursquare'

    authorization_endpoint = 'https://foursquare.com/oauth2/authenticate'
    token_endpoint = 'https://foursquare.com/oauth2/access_token'

    profile_endpoint = 'https://api.foursquare.com/v2/users/self'

    def get_raw_profile(self):
        return self.fetch(self.profile_endpoint, oauth_token=self.access_token, v='20121130')

    def get_profile(self):
        _, profile = super(Foursquare, self).get_profile()
        profile = profile['response']['user']
        return {
            'id': profile['id'],
            'name': profile['firstName'] + ' ' + profile['lastName'],
            'email': profile['contact']['email'],
            'picture': '%s36x36%s' % (profile['photo']['prefix'], profile['photo']['suffix'])
        }, profile


class Instagram(OpenIDConnect):
    name = 'instagram'

    authorization_endpoint = 'https://api.instagram.com/oauth/authorize'
    token_endpoint = 'https://api.instagram.com/oauth/access_token'

    def __init__(self, key, secret, scopes=(), timeout=None):
        super(Instagram, self).__init__(key, secret, scopes, timeout)
        self.user = {}

    def fetch(self, url, post=False, headers=None, **kw):
        data = super(Instagram, self).fetch(url, post=post, headers=headers, **kw)
        if 'user' in data:
            self.user = data['user']
        return data

    def get_profile(self):
        return {
            'id': self.user['id'],
            'name': self.user['full_name'],
            'email': None,
            'picture': self.user['profile_picture']
        }, self.user

"""
# Too limited requests rate

class Reddit(OpenIDConnect):
    name = 'reddit'

    authorization_endpoint = 'https://ssl.reddit.com/api/v1/authorize'
    token_endpoint = 'https://ssl.reddit.com/api/v1/access_token'

    def __init__(self, key, secret, scopes=(), offline=False, prompt=False):
        super(Reddit, self).__init__(key, secret, ('identity',) + scopes)
"""

providers = dict([(provider.name, provider) for provider in (
    Google,
    Twitter,
    Facebook,
    Github,
    Dropbox,
    Salesforce,
    Flickr,
    Vimeo,
    Bitbucket,
    Yahoo,
    Dailymotion,
    Viadeo,
    Linkedin,
    Foursquare,
    Instagram
)])

# -----------------------------------------------------------------------------


class Middleware(object):

    def __init__(self, app, options, config_filename, config, error):
        self.app = app

    def __call__(self, environ, start_response):
        params = dict(urlparse.parse_qsl(environ['QUERY_STRING']))
        if ('state' in params):
            environ['QUERY_STRING'] += ('&' + params['state'])

        return self.app(environ, start_response)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import using_options
from elixir import ManyToOne, ManyToMany
from elixir import Field, Unicode, Integer

from kansha.models import Entity


class DataLabel(Entity):

    """Label mapper
    """
    using_options(tablename='label')
    title = Field(Unicode(255))
    color = Field(Unicode(255))
    board = ManyToOne('DataBoard')
    cards = ManyToMany('DataCard', tablename='label_cards__card_labels')
    index = Field(Integer)

    def copy(self):
        new_data = DataLabel(title=self.title,
                             color=self.color,
                             index=self.index)
        return new_data

    def remove(self, card):
        self.cards.remove(card)

    def add(self, card):
        self.cards.append(card)

    @classmethod
    def get_by_card(cls, card):
        q = cls.query
        q = q.filter(cls.cards.contains(card))
        return q.order_by(cls.index)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--
from base64 import b64encode, b64decode

from Crypto import Random
from peak.rules import when
from Crypto.Cipher import Blowfish
from nagare import security
from nagare.security import form_auth, common

from .card import Card  # Do not remove
from .board import Board  # Do not remove
from .user.usermanager import UserManager
from .column import CardsCounter, Column  # Do not remove
from .board import COMMENTS_PUBLIC, COMMENTS_MEMBERS


class Unauthorized(Exception):
    pass


class Authentication(form_auth.Authentication):

    KEY = 'a*/-+@45JZWiuet:uietsVLMRS/*-+41"_bépçè€.;'

    def _create_user(self, username):
        if username is not None:
            return UserManager.get_app_user(username)

    def check_password(self, username, _, password):
        user = UserManager.get_by_username(username)
        if not user or not user.email:
            return False
        return user.check_password(password)

    def denies(self, detail):
        raise Unauthorized()

    def cookie_decode(self, cookie):
        try:
            a_iv, token = cookie.split(':')
            iv = b64decode(a_iv)
            cipher = Blowfish.new(self.KEY, Blowfish.MODE_CBC, iv)
            cookie = cipher.decrypt(b64decode(token)).replace('@','')
        except ValueError:
            return (None, None)

        return super(Authentication, self).cookie_decode(cookie)

    def cookie_encode(self, *ids):
        cookie = super(Authentication, self).cookie_encode(*ids)

        bs = Blowfish.block_size
        iv = Random.new().read(bs)
        cipher = Blowfish.new(self.KEY, Blowfish.MODE_CBC, iv)
        # pad cookie to block size.
        # Cookie is ascii base64 + ':', so we can safely pad with '@'.
        missing_bytes = bs - (len(cookie) % bs)
        cookie += '@' * missing_bytes

        return '%s:%s' % (
            b64encode(iv),
            b64encode(cipher.encrypt(cookie))
        )

        return cookie


class Rules(common.Rules):

    @when(common.Rules.has_permission, "user is None")
    def _(self, user, perm, subject):
        """Default security, if user is not logged return False"""
        return False

    @when(common.Rules.has_permission, "user and perm == 'view' and isinstance(subject, Board)")
    @when(common.Rules.has_permission, "user is None and perm == 'view'  and isinstance(subject, Board)")
    def _(self, user, perm, board):
        """Test if user can see the board."""
        return board.is_open or (user is not None and board.has_member(user) and not board.archived)

    @when(common.Rules.has_permission, "user and perm == 'manage' and isinstance(subject, Board)")
    def _(self, user, perm, board):
        """Test if users is one of the board's managers"""
        return board.has_manager(user)

    @when(common.Rules.has_permission, "user and (perm == 'edit') and isinstance(subject, Board)")
    def _(self, user, perm, board):
        """Test if users is one of the board's members"""
        return board.has_member(user)

    @when(common.Rules.has_permission, "user and (perm == 'leave') and isinstance(subject, Board)")
    def _(self, user, perm, board):
        """Test if users is one of the board's members"""
        return board.has_member(user)

    @when(common.Rules.has_permission, "user and (perm == 'edit') and isinstance(subject, Column)")
    def _(self, user, perm, column):
        return security.has_permissions('edit', column.board)

    @when(common.Rules.has_permission, "user and (perm == 'edit') and isinstance(subject, Card)")
    def _(self, user, perm, card):
        return card.can_edit(user)

    @when(common.Rules.has_permission, "user and (perm == 'create_board')")
    def _(self, user, perm, subject):
        """If user is logged, he is allowed to create a board"""
        return True

    @when(common.Rules.has_permission, "user and (perm == 'edit') and isinstance(subject, CardsCounter)")
    def _(self, user, perm, CardsCounter):
        return security.has_permissions('edit', CardsCounter.column)


class SecurityManager(Authentication, Rules):

    def __init__(self, crypto_key):
        Authentication.__init__(self)
        Rules.__init__(self)
        self.KEY = crypto_key



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import random

from peak.rules import when
from cgi import FieldStorage
from webob.exc import HTTPOk

from nagare.i18n import _
from nagare.security import common
from nagare import component, security, var

from kansha import validator
from kansha.user import usermanager
from kansha.cardextension import CardExtension
from kansha.services.actionlog.messages import render_event
from kansha.services.assetsmanager import AssetsManager

from .models import DataAsset


IMAGE_CONTENT_TYPES = ('image/png', 'image/jpeg', 'image/pjpeg', 'image/gif')


@when(render_event, "action=='card_add_file'")
def render_event_card_add_file(action, data):
    return _(u'User %(author)s has added file "%(file)s" to card "%(card)s"') % data


class Gallery(CardExtension):

    LOAD_PRIORITY = 40

    def __init__(self, card, action_log, configurator, assets_manager_service):
        """Init method

        In:
            - ``card`` -- card component
        """
        super(Gallery, self).__init__(card, action_log, configurator)
        self.assets_manager = assets_manager_service
        self.assets = []
        self.comp_id = str(random.randint(10000, 100000))
        self.model = 'view'
        self.cropper = component.Component()

    @property
    def num_assets(self):
        return DataAsset.count_for(self.card.data)

    def load_assets(self):
        if not self.assets:
            for asset_data in DataAsset.get_all(self.card.data):
                self.create_asset(asset_data)

    def delete_asset(self, asset):
        """Delete asset

        Delete asset from card (component and overlay), database
        and asset manager.

        In:
            - ``asset`` -- Asset to delete
        """
        for a in self.assets:
            if a() is asset:
                self.assets.remove(a)
                break
        DataAsset.remove(self.card.data, asset.filename)
        # asset.delete()
        self.assets_manager.delete(asset.filename)

    def delete(self):
        self.load_assets()
        for asset in self.assets:
            self.assets_manager.delete(asset().filename)
        DataAsset.remove_all(self.card.data)
        self.assets = []

    def update(self, other):
        other.load_assets()
        for asset_comp in other.assets:
            asset = asset_comp()
            new_asset = self._add_asset(asset.file_info)
            if asset.is_cover:
                cropper = asset.cropper_info
                self.make_cover(new_asset, **cropper)

    def action(self, response):
        action, asset = response[0], response[1]
        if action == 'delete':
            self.delete_asset(asset)
        elif action == 'configure_cover':
            self.configure_cover(asset)
        elif action == 'make_cover':
            left, top, width, height = response[2:]
            self.make_cover(asset, left, top, width, height)
        elif action == 'cancel_cover':
            self.model = 'view'
        elif action == 'remove_cover':
            self.remove_cover(asset)

    def create_asset(self, data_asset):
        asset = Asset(data_asset, self.assets_manager)
        asset_comp = component.Component(asset).on_answer(self.action)
        self.assets.append(asset_comp)
        return asset

    def add_asset(self, new_file):
        """Add one new file to card

        In:
            - ``new_file`` -- new file to add

        Return:
            - The newly created Asset
        """
        file_info = validator.validate_file(new_file, self.assets_manager.max_size, _(u'File must be less than %d KB'))
        return self._add_asset(file_info)

    def _add_asset(self, file_info):
        user = security.get_user()
        fileid = self.assets_manager.save(file_info['data'],
                                          metadata={'filename': file_info['filename'], 'content-type': file_info['content_type']})
        data = {'file': file_info['filename'], 'card': self.card.get_title()}
        self.action_log.add_history(user, u'card_add_file', data)
        return self.create_asset(DataAsset.add(fileid, self.card.data, user.data))

    def add_assets(self, new_files):
        """Add new assets to the card

        In:
            - ``new_files`` -- new files to add
        """
        if isinstance(new_files, FieldStorage):
            self.add_asset(new_files)
        else:
            for new_file in new_files:
                self.add_asset(new_file)

    def configure_cover(self, asset):
        self.model = 'crop'
        self.cropper.call(AssetCropper(asset))

    def make_cover(self, asset, left, top, width, height):
        """Make the given asset as cover for the card

        In :
          - ``asset`` -- The asset to make as cover
          - ``left`` -- Left coordinate of the crop origin
          - ``top`` -- Top coordinate of the crop origin
          - ``width`` -- Crop width
          - ``height`` -- Crop height
        """
        self.assets_manager.create_cover(asset.filename, left, top, width, height)
        DataAsset.set_cover(self.card.data, asset.data)
        for a in self.assets:
            a().is_cover = False
        asset.is_cover = True
        asset.update_metadata(
            {
                'cropper': {
                    'left': left,
                    'top': top,
                    'width': width,
                    'height': height
                }
            }
        )
        self.model = 'view'

    def get_cover(self):
        cover_data = DataAsset.get_cover(self.card.data)
        if not cover_data:
            return None

        return Asset(cover_data, self.assets_manager)

    def remove_cover(self, asset):
        """Don't use the asset as cover anymore

        In :
          - ``asset`` -- The asset
        """
        DataAsset.remove_cover(self.card.data)
        asset.is_cover = False
        self.model = 'view'


class Asset(object):

    def __init__(self, data_asset, assets_manager_service):
        self.filename = data_asset.filename
        self.creation_date = data_asset.creation_date
        self.author = component.Component(usermanager.UserManager.get_app_user(data_asset.author.username, data=data_asset.author))
        self.assets_manager = assets_manager_service
        self.is_cover = data_asset.cover is not None
        self.metadata = self.assets_manager.get_metadata(self.filename)

    @property
    def content_type(self):
        return self.metadata['content-type']

    @property
    def cropper_info(self):
        width, height = AssetsManager.COVER_SIZE
        defaults = {'left': 0, 'top': 0, 'width': width, 'height': height}
        return self.metadata.get('cropper', defaults)

    def update_metadata(self, metadata):
        self.metadata.update(metadata)
        self.assets_manager.update_metadata(self.filename, self.metadata)

    @property
    def file_info(self):
        data, meta_data = self.assets_manager.load(self.filename, None)
        return {
            'data': data,
            'content_type': meta_data['content-type'],
            'filename': meta_data['filename']
        }

    def is_image(self):
        return self.content_type in IMAGE_CONTENT_TYPES

    @property
    def data(self):
        return DataAsset.get_by_filename(self.filename)

    def cancel(self, comp, answer):
        comp.answer()


class AssetCropper(object):

    def __init__(self, asset):
        self.asset = asset
        self.crop_left = var.Var(0)
        self.crop_top = var.Var(0)
        self.crop_width = var.Var(425)
        self.crop_height = var.Var(250)

    def commit(self, comp):
        comp.answer(('make_cover', self.asset, self.crop_left(), self.crop_top(), self.crop_width(), self.crop_height()))

    def remove_cover(self, comp):
        comp.answer(('remove_cover', self.asset))

    def cancel(self, comp):
        comp.answer(('cancel_cover', self.asset))


@when(common.Rules.has_permission, "user and (perm == 'edit') and isinstance(subject, Gallery)")
def has_permission_Gallery_edit(self, user, perm, gallery):
    """Test if description is editable"""
    return security.has_permissions('edit', gallery.card)


# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import json
import os
import uuid

from PIL import Image
from PIL import ImageOps

from nagare import log

from ..assetsmanager import AssetsManager


class SimpleAssetsManager(AssetsManager):

    def __init__(self, config_filename,  error, basedir, baseurl, max_size):
        super(SimpleAssetsManager, self).__init__(config_filename, error)
        self.basedir = basedir
        self.baseurl = baseurl
        self.max_size = max_size

    def copy_cover(self, file_id, new_file_id):
        try:
            data, metadata = self.load(file_id, 'cover')
            with open(self._get_filename(new_file_id, 'cover'), "w") as f:
                f.write(data)
        except IOError:
            # Cover not existing
            pass

    def copy(self, file_id):
        data, metadata = self.load(file_id)
        return self.save(data, metadata=metadata)

    def _get_filename(self, file_id, size=None):
        filename = os.path.join(self.basedir, file_id)
        if size and size != 'large':
            filename += '.' + size
        return filename

    def _get_metadata_filename(self, file_id):
        return os.path.join(self.basedir, '%s.metadata' % file_id)

    def save(self, data, file_id=None, metadata={}, THUMB_SIZE=()):
        if file_id is None:
            file_id = unicode(uuid.uuid4())
        with open(self._get_filename(file_id), "w") as f:
            f.write(data)
        # Store metadata
        with open(self._get_metadata_filename(file_id), "w") as f:
            f.write(json.dumps(metadata))

        img = None
        try:
            img = Image.open(self._get_filename(file_id))
        except IOError:
            log.info('Not an image file, skipping medium & thumbnail generation')
        else:
            # Store thumbnail & medium
            kw = {}
            if 'transparency' in img.info:
                kw['transparency'] = img.info["transparency"]

            orig_width, orig_height = img.size

            medium_size = self.MEDIUM_WIDTH, int(float(self.MEDIUM_WIDTH) * orig_height / orig_width)
            medium = img.copy()

            medium.thumbnail(medium_size, Image.ANTIALIAS)
            medium.save(self._get_filename(file_id, 'medium'), img.format, quality=75, **kw)  # 'JPEG')

            thumb = ImageOps.fit(img, THUMB_SIZE if THUMB_SIZE else self.THUMB_SIZE, Image.ANTIALIAS)
            thumb.save(self._get_filename(file_id, 'thumb'), img.format, quality=75, **kw)  # 'JPEG')
        return file_id

    def delete(self, file_id):
        files = [self._get_filename(file_id),
                 self._get_filename(file_id, 'thumb'),
                 self._get_filename(file_id, 'medium'),
                 self._get_filename(file_id, 'cover'),
                 self._get_filename(file_id, 'large'),
                 self._get_metadata_filename(file_id)]
        for f in files:
            try:
                os.remove(f)
            except OSError:  # File does not exist
                pass

    def load(self, file_id, size=None):
        filename = self._get_filename(file_id, size)
        with open(filename, "r") as f:
            data = f.read()
        return data, self.get_metadata(file_id)

    def update_metadata(self, file_id, metadata):
        with open(self._get_metadata_filename(file_id), "w") as f:
            metadata = f.write(json.dumps(metadata))

    def get_metadata(self, file_id):
        try:
            f = open(self._get_metadata_filename(file_id), "r")
            metadata = json.loads(f.read())
            f.close()
        except IOError:
            log.error('unable to load metadata for ' + self._get_metadata_filename(file_id))
            metadata = {}
        return metadata

    def get_image_size(self, fileid):
        """Return the image dimensions
        In:
            - ``fileid`` -- file identifier
        Return:
            - a tuple representing the image dimensions (width, height) or (None, None) in case of error
        """
        dim = (None, None)
        try:
            dim = Image.open(self._get_filename(fileid)).size
        except:
            log.error('Could not get image dimensions of %r', fileid, exc_info=True)
        return dim

    def get_image_url(self, file_id, size=None, include_filename=True):
        """Return an image significant URL
        In:
            - ``file_id`` -- file identifier
            - ``size`` -- size to get (thumb, medium, cover, large)
            - ``include_filename`` -- add the filename to the URL or not

        Return:
            - image significant URL
        """
        if self.baseurl:
            url = [self.baseurl, self.get_entry_name(), file_id, size or 'large']
        else:
            url = ['', self.get_entry_name(), file_id, size or 'large']
        if include_filename:
            url.append(self.get_metadata(file_id)['filename'])
        return '/'.join(url)

    def create_cover(self, file_id, left, top, width, height):
        """Create the cover version for a file

        In :
          - ``file_id`` -- The asset to make as cover
          - ``left`` -- Left coordinate of the crop origin
          - ``top`` -- Top coordinate of the crop origin
          - ``width`` -- Crop width
          - ``height`` -- Crop height
        """

        # The crop dimensions are given for the medium version.
        # Calculate them for the large version

        medium_img = Image.open(self._get_filename(file_id, 'medium'))
        medium_w, medium_h = medium_img.size

        large_img = Image.open(self._get_filename(file_id))
        large_w, large_h = large_img.size
        kw = {}
        if 'transparency' in large_img.info:
            kw['transparency'] = large_img.info["transparency"]

        left, width = int(float(left) * large_w / medium_w), int(float(width) * large_w / medium_w)
        top, height = int(float(top) * large_h / medium_h), int(float(height) * large_h / medium_h)

        n_img = large_img.crop((left, top, left + width, top + height))
        n_img.thumbnail(self.COVER_SIZE, Image.ANTIALIAS)
        n_img.save(self._get_filename(file_id, 'cover'), large_img.format, quality=75, **kw)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import ManyToOne
from elixir import using_options
from nagare.database import session
from elixir import Field, UnicodeText

from kansha.models import Entity


class DataCardDescription(Entity):
    using_options(tablename='card_description')

    description = Field(UnicodeText, default=u'')
    card = ManyToOne('DataCard', ondelete='cascade')

    def update(self, other):
        self.description = other.description
        session.flush()

    @classmethod
    def new(cls, card):
        """Create and persist."""
        desc = cls(card=card)
        session.add(desc)
        session.flush()
        return desc

    @classmethod
    def get_by_card(cls, card):
        q = cls.query
        q = q.filter_by(card=card)
        return q.first()



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from datetime import date

from peak.rules import when
from nagare.security import common
from nagare.i18n import _, format_date
from nagare import component, security

from kansha.card import Card
from kansha.board import excel_export
from kansha.toolbox import calendar_widget
from kansha.cardextension import CardExtension

from .models import DataCardDueDate


@when(common.Rules.has_permission, "user and perm == 'due_date' and isinstance(subject, Card)")
def has_permission_Card_due_date(self, user, perm, card):
    return security.has_permissions('edit', card)


class DueDate(CardExtension):

    LOAD_PRIORITY = 60

    def __init__(self, card, action_log, configurator):
        """Initialization

        In:
            - ``card`` -- the object card
        """
        super(DueDate, self).__init__(card, action_log, configurator)
        self.due_date = self.get_value()
        self.calendar = calendar_widget.Calendar(self.due_date, allow_none=True)
        self.calendar = component.Component(self.calendar)

    @property
    def data(self):
        data = DataCardDueDate.get_by_card(self.card.data)
        if data is None:
            data = DataCardDueDate(card=self.card.data)
        return data

    def get_value(self):
        return self.data.due_date

    def set_value(self, value):
        '''Set the value to a new date (or None)'''
        self.data.due_date = value
        self.due_date = value

    def new_card_position(self, value):
        self.set_value(value)

    def get_days_count(self):
        today = date.today()
        diff = today - self.due_date
        return diff.days

    def get_class(self):
        '''Get its css class depending on the amount of time between today and the value'''
        if not self.due_date:
            return ''
        diff = self.get_days_count()
        if diff < -1:
            return 'future'
        if diff == -1:
            return 'tomorrow'
        if diff == 0:  # Due date is today
            return 'today'
        if diff == 1:
            return 'yesterday'
        if diff > 1:
            return 'past'
        return ''

    def delete(self):
        self.data.delete()


@excel_export.get_extension_title_for(DueDate)
def get_extension_title_DueDate(card_extension):
    return _(u'Due date')


@excel_export.write_extension_data_for(DueDate)
def write_extension_data_DueDate(self, sheet, row, col, style):
    value = self.get_value()
    sheet.write(row, col, format_date(value) if value else u'', style)



#-*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--


"""
This engine does nothing but it works (i.e. it does not break)
It is here for documentation purpose.
For a basic effective engine, see SQLiteFTSEngine.
"""


class DummySearchEngine(object):

    # Make it compatible with services
    LOAD_PRIORITY = 30

    def __init__(self, index):
        self.index = index

    def add_document(self, document):
        '''
        Add a document to the data store, in index (a.k.a. collection)
        `index`, under the document type (a.k.a. schema) `schema`.
        The document is a `schema.Document` instance.
        '''
        pass

    def delete_document(self, schema, docid):
        '''
        Remove document from index and storage.
        schema is a `Document` class or Schema object.
        '''
        pass

    def update_document(self, document):
        '''Update document'''
        pass

    def commit(self, sync=False):
        '''
        Validate add/upgrade/delete operations since last commit.
        The ``sync`` option controls whether the operation is
        synchronous or not (default to False) if backend supports it.
        '''
        pass

    def cancel(self):
        '''
        Forget documents added since last commit'''
        pass

    def search(self, query, size=20):
        '''
        Search the database.
        The query Query expression (see search.query).
        Return list of tuples (score, document).
        The returned document only contains stored values, the others
        are set to None.
        '''
        return []

    def create_collection(self, schemas):
        '''
        Init the collections the first time.
        Just use once! Or you'll have to reindex all your documents.
        `schemas` is a list of Document classes or Schema instances.
        '''
        pass

    def delete_collection(self):
        pass



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import unittest
import sys

from nagare import database, i18n, security, component
from nagare.namespaces import xhtml5
from elixir import metadata as __metadata__

from kansha.board import comp as board_module
from kansha.board import boardsmanager
from kansha import helpers
from kansha.security import Unauthorized

database.set_metadata(__metadata__, 'sqlite:///:memory:', False, {})


class BoardTest(unittest.TestCase):

    def setUp(self):
        helpers.setup_db(__metadata__)
        services = helpers.create_services()
        self.boards_manager = boardsmanager.BoardsManager('', '', '', {}, None, services)

    def tearDown(self):
        helpers.teardown_db(__metadata__)

    def test_view_board_1(self):
        """Test security view board 1

        Board Private
        User not logged
        """
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        helpers.set_context()  # real security manager for tests
        self.assertFalse(security.has_permissions('view', board))

    def test_view_board_2(self):
        """Test security view board 2

        Board Public
        User not logged
        """
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        board.set_visibility(board_module.BOARD_PUBLIC)
        user = helpers.create_user('bis')
        helpers.set_context(user)
        self.assertTrue(security.has_permissions('view', board))

    def test_view_board_3(self):
        """Test security view board 3

        Board Private
        User logged but not member of the board
        """
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        board.set_visibility(board_module.BOARD_PRIVATE)
        user = helpers.create_user('bis')
        helpers.set_context(user)
        self.assertFalse(security.has_permissions('view', board))

    def test_view_board_4(self):
        """Test security view board 4

        Board Private
        User member of the board
        """
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        board.set_visibility(board_module.BOARD_PRIVATE)
        user = helpers.create_user('bis')
        helpers.set_context(user)
        board.add_member(user)
        self.assertTrue(security.has_permissions('view', board))

    def test_view_board_5(self):
        """Test security view board 5

        Board Public
        User member of the board
        """
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        board.set_visibility(board_module.BOARD_PUBLIC)
        user = helpers.create_user('bis')
        helpers.set_context(user)
        board.add_member(user)
        self.assertTrue(security.has_permissions('view', board))

    def test_rendering_security_view_board_1(self):
        """Test rendering security view board 1

        Test rendering (Board private / user not logged)
        """
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        board.set_visibility(board_module.BOARD_PRIVATE)
        helpers.set_context()

        with self.assertRaises(Unauthorized):
            component.Component(board).on_answer(lambda x: None).render(xhtml5.Renderer())

    def test_rendering_security_view_board_2(self):
        """Test rendering security view board 2

        Test rendering (Board private / user member)
        """
        # stackless only
        if 'Stackless' not in sys.version:
            return
        helpers.set_dummy_context()  # to be able to create board
        board = helpers.create_board()
        board.set_visibility(board_module.BOARD_PRIVATE)
        user = helpers.create_user('bis')
        helpers.set_context(user)

        board.add_member(user)

        with i18n.Locale('en', 'US'):
            component.Component(board).on_answer(lambda x: None).render(xhtml5.Renderer())

    def test_rendering_security_view_board_3(self):
        """Test rendering security view board 3

        Test rendering (Board public / user not logged)
        """
        helpers.set_dummy_context()  # for board creation
        board = helpers.create_board()
        helpers.set_context()  # for realistic security check
        board.set_visibility(board_module.BOARD_PUBLIC)

        with i18n.Locale('en', 'US'):
            component.Component(board).on_answer(lambda x: None).render(xhtml5.Renderer())



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--
from nagare import component

from kansha import validator


class Title(object):
    def __init__(self, title, class_='title', height=None, placeholder=''):
        self.title = title
        self.class_ = class_
        self.height = height
        self.placeholder = placeholder

    def set_title(self, comp, title):
        text = title() and title().strip()
        if text:
            text = validator.clean_text(text)
            comp.answer(text)


class EditableTitle(component.Task):
    def __init__(self, title, cls=Title, *args, **kw):
        self.title = component.Component(cls(title, *args, **kw))

    def set_title(self, title):
        self.title().title = title

    def go(self, comp):
        while True:
            comp.call(self.title)
            new_title = comp.call(self.title, model='edit')
            if new_title is not None:
                comp.answer(new_title)



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Add notify value

Revision ID: 1b6fbaffad32
Revises: d0e88000d3f
Create Date: 2013-11-04 09:59:21.968436

"""

# revision identifiers, used by Alembic.
revision = '1b6fbaffad32'
down_revision = 'd0e88000d3f'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('user_boards__board_members',
                  sa.Column('notify', sa.Integer, server_default='1'))


def downgrade():
    op.drop_column('user_boards__board_members', 'notify')
